import {
  ChartHelper, FilterType,
  Indicator, IndicatorFilterUtils,
  IndicatorPath, IndicatorPathType, IndicatorType,
  SourceType,
  Stakeholder,
  SubCategory,
  Topic
} from "../openaireLibrary/monitor/entities/stakeholder";
import {AbstractControl, ValidatorFn, Validators} from "@angular/forms";
import {Option} from "../openaireLibrary/dashboard/sharedComponents/input/input.component";

export class StakeholderUtils {
  defaultProfiles = {"funder":{
      index_id:"ec__________::EC",
  index_name:     "European Commission", index_shortName:"EC"}};
  
  
  types: Option[] = [
    {value: 'funder', label: 'Funder'},
    {value: 'ri', label: 'Reasearch Initiative'},
    {value: 'project', label: 'Project'},
    {value: 'organization', label: 'Organization'}
  ];

  isPublic: Option[] = [
    {icon: 'public', value: true, label: 'Public'},
    {icon: 'lock', value: false, label: 'Private'},
  ];

  isActive: Option[] = [
    {icon: 'brightness_1', iconClass: '', value: true, label: 'Active'},
    {icon: 'brightness_1', value: false, label: 'Inactive'},
  ];

  isPublicIcon: Map<boolean, string> = new Map([
    [true, 'public'],
    [false, 'lock']
  ]);

  isActiveIcon: string = 'brightness_1';

  public createFunderFromDefaultProfile(funder: Stakeholder, defaultTopics: Topic[]): Stakeholder {
    funder.topics = defaultTopics;
    for (let topic of funder.topics) {
      // console.log('id:' + topic._id);
      topic.defaultId = topic._id;
      topic._id = null;
      // console.log('defaultId:' + topic.defaultId);
      for (let category of topic.categories) {
        category.defaultId = category._id;
        category._id = null;
        let subTokeep: SubCategory[] = [];
        for (let subCategory of category.subCategories) {
          subCategory.defaultId = subCategory._id;
          subCategory._id = null;
          if (!subCategory.recommendedFor || subCategory.recommendedFor.length == 0 || subCategory.recommendedFor.indexOf(funder.index_id) != -1) {
            subTokeep.push(subCategory);
          }
          for (let section of subCategory.charts) {
            let chartsTokeep: Indicator[] = [];
            section.defaultId = section._id;
            section.stakeholderAlias = funder.alias;
            section._id = null;
            for (let indicator of section.indicators) {
              indicator.defaultId = indicator._id;
              indicator._id = null;
              if (!indicator.recommendedFor || indicator.recommendedFor.length == 0 || indicator.recommendedFor.indexOf(funder.index_id) != -1) {
                chartsTokeep.push(indicator);
              }
              for (let indicatorPath of indicator.indicatorPaths) {
                if (indicatorPath.parameters) {
                  Object.keys(indicatorPath.parameters).forEach(key => {
                    //TODO check before delete
                    /*if (indicatorPath.parameters[key].indexOf("_funder_name_") != -1) {
                      indicatorPath.parameters[key] = indicatorPath.parameters[key].replace("_funder_name_", funder.index_name);
                    } else if (indicatorPath.parameters[key].indexOf("_funder_id_") != -1) {
                      indicatorPath.parameters[key] = indicatorPath.parameters[key].replace("_funder_id_", funder.index_id);
                    } else if (indicatorPath.parameters[key].indexOf("_fsn_") != -1) {
                      indicatorPath.parameters[key] = indicatorPath.parameters[key].toString().replace("_fsn_", funder.index_shortName.toLowerCase());
                    }*/
                    if (key == "index_name") {
                      indicatorPath.parameters[key] = funder.index_name;
                    } else if (key == "index_id" ) {
                      indicatorPath.parameters[key] = funder.index_id;
                    }  else if (key == "index_shortName" ) {
                      indicatorPath.parameters[key] =  funder.index_shortName.toLowerCase();
                    }
                  });
                }
              }
            }
            section.indicators = chartsTokeep;
          }
          for (let section of subCategory.numbers) {
            section.defaultId = section._id;
            section.stakeholderAlias = funder.alias;
            section._id = null;
            for(let indicator of section.indicators) {
              indicator.defaultId = indicator._id;
              indicator._id = null;
              for (let indicatorPath of indicator.indicatorPaths) {
               /* indicatorPath.url = indicatorPath.url.replace("index_id", encodeURIComponent(funder.index_id));
                indicatorPath.url = indicatorPath.url.replace("index_name",  encodeURIComponent(funder.index_name));
                indicatorPath.url = indicatorPath.url.replace("index_shortName", encodeURIComponent(funder.index_shortName));*/
                // if(indicatorPath.parameters) {
                //   indicatorPath.parameters.forEach((value: string, key: string) => {
                //     if (value.indexOf("_funder_name_")!=-1) {
                //       indicatorPath.parameters.set(key,value.toString().replace("_funder_name_", funder.index_name));
                //     }else if (value.indexOf("_fsn_")!=-1) {
                //       indicatorPath.parameters.set(key,value.toString().replace("_fsn_", funder.index_shortName.toLowerCase()));
                //     }
                //   });
                // }
              }
            }
          }

        }
        category.subCategories = subTokeep;
      }
    }
    console.log (funder)
    return funder;
  }

  aliasValidator(elements: any[]): ValidatorFn {
    return (control: AbstractControl): { [key: string]: boolean } | null => {
      if (control.value && elements.find(element =>
        element.alias === control.value
      )) {
        return {'alias': true};
      }
      return null;
    }
  }
  
  generateAlias(name: string): string {
    let alias = name.toLowerCase();
    while (alias.includes('/') || alias.includes(' ')) {
      alias = alias.replace(' / ', '-');
      alias = alias.replace('/', '-');
      alias = alias.replace(' ', '-');
    }
    return alias;
  }
}

export class IndicatorUtils {

  allChartTypes: Option[] = [
    {value: 'pie', label: 'Pie'},
    {value: 'table', label: 'Table'},
    {value: 'line', label: 'Line'},
    {value: 'column', label: 'Column'},
    {value: 'bar', label: 'Bar'},
    {value: 'other', label: 'Other'}
  ];
  basicChartTypes:IndicatorPathType[] =["pie", "line", "column", "bar"];
  defaultChartType:IndicatorPathType = "other";
  indicatorSizes: Option[] = [
    {value: 'small', label: 'Small'},
    {value: 'medium', label: 'Medium'},
    {value: 'large', label: 'Large'}
  ];

  sourceTypes: Option[]  = [
    {value: 'search', label: 'Search'},
    {value: 'statistics', label: 'Statistics'},
    {value: 'metrics', label: 'Metrics'},
    {value: 'stats-tool', label: 'Statistics tool'}
  ];
  
  isPublic: Option[] = [
    {icon: 'public', value: true, label: 'Public'},
    {icon: 'lock', value: false, label: 'Private'},
  ];

  isActive: Option[] = [
    {icon: 'brightness_1', iconClass: '', value: true, label: 'Active'},
    {icon: 'brightness_1', value: false, label: 'Inactive'},
  ];

  chartTypesIcons: Map<string, string> = new Map([
    ['pie', 'pie_chart'],
    ['table', 'table_chart'],
    ['line', 'show_chart'],
    ['column', 'bar_chart'],
    ['bar', 'notes'],
    ['other', 'perm_media']
  ]);
  getChartTypes(initialType){
    let types: Option[]= [];
    if(this.basicChartTypes.indexOf(initialType) != -1){
      (this.allChartTypes).forEach(option => {
        if(this.basicChartTypes.indexOf(option.value)!=-1){
          types.push(option);
        }
      });
      return types;
    }else if(initialType == "table") {
      (this.allChartTypes).forEach(option => {
        if (initialType == option.value) {
          types.push(option);
        }
      });
      return types;
    }else {
      return this.allChartTypes;
    }
  }
  isPublicIcon: Map<boolean, string> = new Map([
    [true, 'public'],
    [false, 'lock']
  ]);

  isActiveIcon: string = 'brightness_1';

  ignoredParameters = ['index_name','index_id','index_shortName'];

  parametersValidators: Map<string, any> = new Map<string, any>([
    ['start_year', [Validators.required, Validators.pattern('^\\d+$')]],
    ['end_year', [Validators.required, Validators.pattern('^\\d+$')]]
  ]);

  public getFullUrl(stakeholder:Stakeholder, indicatorPath: IndicatorPath, fundingL0: string = null, startYear: string = null, endYear: string = null): string {
    let replacedUrl = indicatorPath.chartObject;
    if (indicatorPath.parameters) {
      Object.keys(indicatorPath.parameters).forEach(key => {
        let replacedValue = indicatorPath.parameters[key];
        if (startYear && key == "start_year" && indicatorPath.filters["start_year"]) {
          replacedValue = (replacedValue < startYear) ? startYear : replacedValue;
        }
        if (endYear && key == "end_year" && indicatorPath.filters["end_year"]) {
          replacedValue = (replacedValue > endYear) ? endYear : replacedValue;
        }
        if (key == "index_id") {
          replacedValue = stakeholder.index_id;
        }
        if (key == "index_name") {
          replacedValue = stakeholder.index_name;
        }
        if (key == "index_shortName") {
          replacedValue = stakeholder.index_shortName.toLowerCase();
        }

        replacedUrl = replacedUrl.split(ChartHelper.prefix + key + ChartHelper.suffix).join(replacedValue)
      });
    }
    if (indicatorPath.chartObject) {
      if (fundingL0 && indicatorPath.filters["fundingL0"]) {
        let newJsonObject = JSON.parse(replacedUrl);
        for (let queries of this.getQueryObjectName(newJsonObject)?newJsonObject[this.getDescriptionObjectName(newJsonObject)][this.getQueryObjectName(newJsonObject)]:newJsonObject[this.getDescriptionObjectName(newJsonObject)]) {
          if (!queries["query"]["filters"] || queries["query"]["filters"].length == 0) {
            queries["query"]["filters"] = [];
          }
          //TODO check how it works if the query already has a filter
          queries["query"]["filters"].push(JSON.parse(indicatorPath.filters["fundingL0"].replace(ChartHelper.prefix + "fundingL0" + ChartHelper.suffix, fundingL0)));
        }
        replacedUrl = JSON.stringify(newJsonObject);
      }
      if (startYear && indicatorPath.filters["start_year"]) {
        let newJsonObject = JSON.parse(replacedUrl);

        for (let queries of this.getQueryObjectName(newJsonObject)?newJsonObject[this.getDescriptionObjectName(newJsonObject)][this.getQueryObjectName(newJsonObject)]:newJsonObject[this.getDescriptionObjectName(newJsonObject)]) {
          if (!queries["query"]["filters"] || queries["query"]["filters"].length == 0) {
            queries["query"]["filters"] = [];
          }
          //TODO check how it works if the query already has a filter
          queries["query"]["filters"].push(JSON.parse(indicatorPath.filters["start_year"].replace(ChartHelper.prefix + "start_year" + ChartHelper.suffix, startYear)));
        }
        replacedUrl = JSON.stringify(newJsonObject);
      }
      if (endYear && indicatorPath.filters["end_year"]) {
        let newJsonObject = JSON.parse(replacedUrl);
        for (let queries of this.getQueryObjectName(newJsonObject)?newJsonObject[this.getDescriptionObjectName(newJsonObject)][this.getQueryObjectName(newJsonObject)]:newJsonObject[this.getDescriptionObjectName(newJsonObject)]) {
          if (!queries["query"]["filters"] || queries["query"]["filters"].length == 0) {
            queries["query"]["filters"] = [];
          }
          //TODO check how it works if the query already has a filter
          queries["query"]["filters"].push(JSON.parse(indicatorPath.filters["end_year"].replace(ChartHelper.prefix + "end_year" + ChartHelper.suffix, endYear)));
        }
        replacedUrl = JSON.stringify(newJsonObject);
      }

    }
    //For numbers
    if(indicatorPath.url.indexOf(ChartHelper.prefix + 'index_id' + ChartHelper.suffix) !=- 1){
      indicatorPath.url = replacedUrl.split(ChartHelper.prefix + 'index_id' + ChartHelper.suffix).join(encodeURIComponent(stakeholder.index_id))
    }
    if(indicatorPath.url.indexOf(ChartHelper.prefix + 'index_name' + ChartHelper.suffix) !=- 1){
      indicatorPath.url = replacedUrl.split(ChartHelper.prefix + 'index_name' + ChartHelper.suffix).join(encodeURIComponent(stakeholder.index_name))
    }
    if(indicatorPath.url.indexOf(ChartHelper.prefix + 'index_shortName' + ChartHelper.suffix) !=- 1){
      indicatorPath.url = replacedUrl.split(ChartHelper.prefix + 'index_shortName' + ChartHelper.suffix).join(encodeURIComponent(stakeholder.index_shortName))
    }
    return indicatorPath.url + encodeURIComponent(replacedUrl);
  }
  public getFullUrlWithFilters(stakeholder:Stakeholder, indicatorPath: IndicatorPath, fundingL0: string = null, startYear: string = null, endYear: string = null, isNumber:boolean=false): string {
    let replacedUrl = indicatorPath.chartObject?indicatorPath.chartObject:indicatorPath.url;
    if (indicatorPath.parameters) {
      Object.keys(indicatorPath.parameters).forEach(key => {
        let replacedValue = indicatorPath.parameters[key];
        if (startYear && key == "start_year") {
          replacedValue = (replacedValue < startYear) ? startYear : replacedValue;
        }
        if (endYear && key == "end_year") {
          replacedValue = (replacedValue > endYear) ? endYear : replacedValue;
        }
        if (key == "index_id") {
          replacedValue = stakeholder.index_id;
        }
        if (key == "index_name") {
          replacedValue = stakeholder.index_name;
        }
        if (key == "index_shortName") {
          replacedValue = stakeholder.index_shortName.toLowerCase();
        }

        replacedUrl = replacedUrl.split(ChartHelper.prefix + key + ChartHelper.suffix).join(replacedValue)
      });
    }
    if (fundingL0) {
      if(indicatorPath.source == "stats-tool" && indicatorPath.chartObject) {
        let filterResults = this.addFilter(replacedUrl, 'fundingL0', fundingL0);
        replacedUrl = filterResults.url;
        indicatorPath.filteredApplied = indicatorPath.filteredApplied|| filterResults.filteredApplied;
      }
    }
    if (startYear) {
      if(indicatorPath.source == "stats-tool" && indicatorPath.chartObject) {
        let filterResults =  this.addFilter(replacedUrl, 'start_year', startYear);
        replacedUrl = filterResults.url;
        indicatorPath.filteredApplied = indicatorPath.filteredApplied|| filterResults.filteredApplied;
      }
    }
    if (endYear ) {
      if(indicatorPath.source == "stats-tool" && indicatorPath.chartObject) {
        let filterResults = this.addFilter(replacedUrl, 'end_year', endYear);
        replacedUrl = filterResults.url;
        indicatorPath.filteredApplied = indicatorPath.filteredApplied|| filterResults.filteredApplied;
      }
    }

    //For numbers
    if (replacedUrl.indexOf(ChartHelper.prefix + 'index_id' + ChartHelper.suffix) != -1) {
      replacedUrl = replacedUrl.split(ChartHelper.prefix + 'index_id' + ChartHelper.suffix).join(encodeURIComponent(stakeholder.index_id))
    }
    if (replacedUrl.indexOf(ChartHelper.prefix + 'index_name' + ChartHelper.suffix) != -1) {
      replacedUrl = replacedUrl.split(ChartHelper.prefix + 'index_name' + ChartHelper.suffix).join(encodeURIComponent(stakeholder.index_name))
    }
    if (replacedUrl.indexOf(ChartHelper.prefix + 'index_shortName' + ChartHelper.suffix) != -1) {
      replacedUrl = replacedUrl.split(ChartHelper.prefix + 'index_shortName' + ChartHelper.suffix).join(encodeURIComponent(stakeholder.index_shortName))
    }

    return (indicatorPath.chartObject?indicatorPath.url + encodeURIComponent(replacedUrl):replacedUrl);

  }
  private  addFilter(replacedUrl, filterType:FilterType, filterValue){
    let newJsonObject = JSON.parse(replacedUrl);
    let filterApplied:boolean = false;
    for (let queries of this.getQueryObjectName(newJsonObject)?newJsonObject[this.getDescriptionObjectName(newJsonObject)][this.getQueryObjectName(newJsonObject)]:newJsonObject[this.getDescriptionObjectName(newJsonObject)]) {

      if(queries["query"]["name"] && !queries["query"]["select"]){
        /*
        //Uncomment when the named queries filters are defined
        //monitor.stakeholderType.name.id.type.fl0
        if(queries["query"]["name"].indexOf("monitor.")==-1){
          continue;
        }
        filterApplied = true;
        let filterPosition = queries["query"]["name"].split(".").indexOf(filterType=="fundingL0"?'fl0':filterType);
        if(filterType == "fundingL0"){
          filterApplied = true;
          if(filterPosition != -1){
            queries["query"]["parameters"][filterPosition-3]=filterValue;
          }else{
            queries["query"]["name"]=queries["query"]["name"]+".fl0";
            queries["query"]["parameters"].push(filterValue);
          }
        }else {
          filterApplied = true;
          //start_year end_year
          if(filterPosition != -1){
            queries["query"]["parameters"][filterPosition-3]=filterValue;
          }else{
            queries["query"]["name"]=queries["query"]["name"]+".start_year.end_year";
            filterPosition = queries["query"]["name"].split(".").indexOf(filterType);
            queries["query"]["parameters"].push("0");
            queries["query"]["parameters"].push("9999");
            queries["query"]["parameters"][filterPosition-3]=filterValue
          }
        }
        console.debug(queries["query"])*/
        // it is a name query
        continue;
      }
      if (!queries["query"]["filters"] || queries["query"]["filters"].length == 0) {
        queries["query"]["filters"] = [];
      }
      let field = queries["query"]["select"][0]["field"];
      // console.debug("Field is"+field)
      let filterString =  IndicatorFilterUtils.getFilter(field,filterType);
      if(filterString){
        filterApplied = true;
        let filter = JSON.parse(filterString);
        let filterposition = IndicatorFilterUtils.filterIndexOf(filter,queries["query"]["filters"]);
        if(filterposition){
          queries["query"]["filters"][filterposition.filter][filterposition.groupFilter] = filter['groupFilters'][0]["values"][0].replace(ChartHelper.prefix + filterType + ChartHelper.suffix, filterValue);
        }
        queries["query"]["filters"].push(JSON.parse(filterString.replace(ChartHelper.prefix + filterType + ChartHelper.suffix, filterValue)));
      }
    }
    return  { "url":JSON.stringify(newJsonObject), "filteredApplied":filterApplied};
  }

  generateIndicatorByForm(form: any, indicatorPaths: IndicatorPath[], type:IndicatorType, addParameters:boolean = true ): Indicator {
    let indicator: Indicator = new Indicator(form.name, form.description, type,
      form.width, form.isActive, form.isPublic, indicatorPaths, form.defaultId);
    indicator._id = form._id;
    form.indicatorPaths.forEach((indicatorPath, index) => {
      indicator.indicatorPaths[index].type = indicatorPath.type;
      if(addParameters) {
        indicatorPath.parameters.forEach(parameter => {
          indicator.indicatorPaths[index].parameters[parameter.key] = parameter.value;
          if (parameter.key === 'type') {
            indicator.indicatorPaths[index].type = parameter.value;
          }
        });
      }
    });
    return indicator;
  }
  generateIndicatorByNumberUrl(source: SourceType, url: string, stakeholder:Stakeholder, jsonPath = []): IndicatorPath {
    let indicatorPath = new IndicatorPath(null, source, url, null, jsonPath);
    if (source === 'stats-tool') {
      indicatorPath.url = url.split("json=")[0] + "json=";
      indicatorPath.url = indicatorPath.url.split("/")[indicatorPath.url.split("/").length - 1];
      indicatorPath.chartObject = decodeURIComponent(url.indexOf("json=")!=-1?url.split("json=")[1]:"");
      let chart = JSON.parse(indicatorPath.chartObject);
      this.extractStakeHolders(chart, indicatorPath, stakeholder);
      // this.addResultFilters(chart, indicatorPath);
    }else {
      try {
        if (url.indexOf(encodeURIComponent(stakeholder.index_id)) !== -1) {
          url = url.split(encodeURIComponent(stakeholder.index_id)).join(ChartHelper.prefix + "index_id" + ChartHelper.suffix);
        }
        if (url.indexOf(encodeURIComponent(stakeholder.index_name)) !== -1) {
          url = url.split(encodeURIComponent(stakeholder.index_name)).join(ChartHelper.prefix + "index_name" + ChartHelper.suffix);
        }
        if (url.indexOf(encodeURIComponent(stakeholder.index_shortName)) !== -1) {
          url = url.split(encodeURIComponent(stakeholder.index_shortName)).join(ChartHelper.prefix + "index_shortName" + ChartHelper.suffix);
        }
        indicatorPath.url = url;
      } catch (e) {
        console.error(e);
      }
    }
    return indicatorPath;
  }
  generateIndicatorByChartUrl(source: SourceType, url: string, type: IndicatorPathType = null, stakeholder:Stakeholder): IndicatorPath {
    let indicatorPath = new IndicatorPath(type, source, "", "", []);
    try {
      if (source === 'stats-tool') {
        indicatorPath.url = url.split("json=")[0] + "json=";
        indicatorPath.url = indicatorPath.url.split("/")[indicatorPath.url.split("/").length - 1];
        indicatorPath.chartObject = decodeURIComponent(url.split("json=")[1]);
        let chart = JSON.parse(indicatorPath.chartObject);
        // console.debug(indicatorPath);
        if (indicatorPath.url == "chart?json=") {

          if (chart["library"] && (chart["library"] == "HighCharts" || chart["library"] == "eCharts" || chart["library"] == "HighMaps" )) {
            indicatorPath.type = this.extractType(chart, indicatorPath);
          } else {
            indicatorPath.type = this.defaultChartType;
          }

          this.extractTitle(chart, indicatorPath);
          this.extractSubTitle(chart, indicatorPath);
          this.extractXTitle(chart, indicatorPath);
          this.extractYTitle(chart, indicatorPath);
        }else if(indicatorPath.url == "table?json="){
          indicatorPath.type = "table";
        }
        if (indicatorPath.url == "chart?json=" || indicatorPath.url == "table?json=") {
          // common for tables and other chart types
          this.extractDataTitle(chart, indicatorPath);
          this.parameterizeDefaultQuery(chart, indicatorPath, stakeholder);
          this.extractStakeHolders(chart, indicatorPath, stakeholder);
          this.extractStartYear(chart, indicatorPath);
          this.extractEndYear(chart, indicatorPath);
          indicatorPath.chartObject = JSON.stringify(chart);
        }
      } else if (source === 'old') {
        indicatorPath.url = url.split("data=")[0].split("/stats/")[1] + "data=";
        indicatorPath.chartObject = decodeURIComponent(url.split("data=")[1].split("&")[0]);
        indicatorPath.type = type;
        let chart = JSON.parse(indicatorPath.chartObject);
        this.extractOldToolTitle(chart, indicatorPath);
        this.extractOldToolXTitle(chart, indicatorPath);
        this.extractOldToolYTitle(chart, indicatorPath);
        indicatorPath.chartObject = JSON.stringify(chart);
      } else {
        indicatorPath.url = url;
        indicatorPath.type = type;
      }
    }catch(e){
      console.error(e);
      indicatorPath.url = url;
      indicatorPath.type = type;
    }
    // console.debug(indicatorPath.parameters);
    // console.debug(indicatorPath.chartObject);
    if(indicatorPath.type == null){
      indicatorPath.type = this.defaultChartType;
    }
    return indicatorPath;
  }
  private getQueryObjectName(obj){
    if((obj[this.getDescriptionObjectName(obj)]).hasOwnProperty("queriesInfo")){
        return "queriesInfo";
    }else if((obj[this.getDescriptionObjectName(obj)]).hasOwnProperty("queries")) {
      return "queries";
    }
  }
  private getDescriptionObjectName(obj){
    if(obj.hasOwnProperty("mapDescription")){
      return "mapDescription";
    }else if(obj.hasOwnProperty("chartDescription")) {
      return "chartDescription";
    }else if(obj.hasOwnProperty("tableDescription") ){
      return "tableDescription";
    }else if(obj.hasOwnProperty("series") ){
      return "series";
    }
    }
  private extractType(obj, indicatorPath: IndicatorPath): IndicatorPathType {
    let type = (obj[this.getDescriptionObjectName(obj)] && obj[this.getDescriptionObjectName(obj)][this.getQueryObjectName(obj)][0]["type"])?obj[this.getDescriptionObjectName(obj)][this.getQueryObjectName(obj)][0]["type"]:"";
    if (this.basicChartTypes.indexOf(type) == -1) {
      type = this.defaultChartType;
    } else {
      obj[this.getDescriptionObjectName(obj)]["queries"][0]["type"] = ChartHelper.prefix + "type" + ChartHelper.suffix;
      indicatorPath.parameters['type'] = type;
    }
    return type;
  }
  private extractStakeHolders(obj, indicatorPath: IndicatorPath, stakeholder:Stakeholder) {
    this.extractFunder(obj, indicatorPath, stakeholder);
    this.extractRI(obj, indicatorPath, stakeholder);
    this.extractOrganization(obj, indicatorPath, stakeholder);
  }
  private extractFunder(obj, indicatorPath: IndicatorPath, stakeholder:Stakeholder) {
    if(stakeholder.type != "funder"){
      return;
    }
    for (let query of this.getQueryObjectName(obj)?obj[this.getDescriptionObjectName(obj)][this.getQueryObjectName(obj)]:obj[this.getDescriptionObjectName(obj)]) {
      if (!query["query"]["filters"]) {
        return;
      }
      for (let filter of query["query"]["filters"]) {
        if (filter["groupFilters"][0]["field"].indexOf(".funder") != -1) {
          filter["groupFilters"][0]["values"][0] = ChartHelper.prefix + "index_name" + ChartHelper.suffix;
          indicatorPath.parameters["index_name"] = stakeholder.index_name;
        }else  if (filter["groupFilters"][0]["field"].indexOf(".funder.id") != -1) {
          filter["groupFilters"][0]["values"][0] = ChartHelper.prefix + "index_shortName" + ChartHelper.suffix;
          indicatorPath.parameters["index_shortName"] = stakeholder.index_shortName;
        }
      }
    }
  }

  private extractRI(obj, indicatorPath: IndicatorPath, stakeholder:Stakeholder) {
    if(stakeholder.type != "ri"){
      return;
    }
    for (let query of obj[this.getDescriptionObjectName(obj)][this.getQueryObjectName(obj)]) {
      if (!query["query"]["filters"]) {
        return;
      }
      for (let filter of query["query"]["filters"]) {
        if (filter["groupFilters"][0]["field"].indexOf(".context.name") != -1) {
          filter["groupFilters"][0]["values"][0] = ChartHelper.prefix + "index_name" + ChartHelper.suffix;
          indicatorPath.parameters["index_name"] = stakeholder.index_name;
        }else  if (filter["groupFilters"][0]["field"].indexOf(".context.id") != -1) {
          filter["groupFilters"][0]["values"][0] = ChartHelper.prefix + "index_shortName" + ChartHelper.suffix;
          indicatorPath.parameters["index_shortName"] = stakeholder.index_shortName;
        }
      }
    }
  }

  private extractOrganization(obj, indicatorPath: IndicatorPath, stakeholder:Stakeholder) {
    // works for publication.project.organization.name
    // and publication.organization.name
    if(stakeholder.type != "organization"){
      return;
    }
    for (let query of obj[this.getDescriptionObjectName(obj)][this.getQueryObjectName(obj)]) {
      if (!query["query"]["filters"]) {
        return;
      }
      for (let filter of query["query"]["filters"]) {
        if (filter["groupFilters"][0]["field"].indexOf(".organization.name") != -1) {
          filter["groupFilters"][0]["values"][0] = ChartHelper.prefix + "index_name" + ChartHelper.suffix;
          indicatorPath.parameters["index_name"] = stakeholder.index_name;
        }else  if (filter["groupFilters"][0]["field"].indexOf(".organization.id") != -1) {
          filter["groupFilters"][0]["values"][0] = ChartHelper.prefix + "index_shortName" + ChartHelper.suffix;
          indicatorPath.parameters["index_shortName"] = stakeholder.index_shortName;
        }
      }
    }
  }
  private extractStartYear(obj, indicatorPath: IndicatorPath) {
    let start_year;
    for (let query of obj[this.getDescriptionObjectName(obj)][this.getQueryObjectName(obj)]) {
      if (!query["query"]["filters"]) {
        return;
      }
      for (let filter of query["query"]["filters"]) {
        for (let gfilter of filter["groupFilters"]) {
          if (gfilter["field"].indexOf(".year") != -1 && gfilter["type"].indexOf(">") != -1) {
            start_year = gfilter["values"][0];
            gfilter["values"][0] = ChartHelper.prefix + "start_year" + ChartHelper.suffix;
            indicatorPath.parameters["start_year"] = start_year;
          }
        }
      }
    }
  }

  private extractEndYear(obj, indicatorPath: IndicatorPath) {
    let end_year;
    for (let query of obj[this.getDescriptionObjectName(obj)][this.getQueryObjectName(obj)]) {
      if (!query["query"]["filters"]) {
        return;
      }
      for (let filter of query["query"]["filters"]) {
        for (let gfilter of filter["groupFilters"]) {
          if (gfilter["field"].indexOf(".year") != -1 && gfilter["type"].indexOf("<") != -1) {
            end_year = gfilter["values"][0];
            gfilter["values"][0] = ChartHelper.prefix + "end_year" + ChartHelper.suffix;
            indicatorPath.parameters["end_year"] = end_year;
          }
        }
      }
    }
  }

  private parameterizeDefaultQuery(obj, indicatorPath: IndicatorPath, stakeholder:Stakeholder) {
    let name = "";
    for (let query of obj[this.getDescriptionObjectName(obj)][this.getQueryObjectName(obj)]) {
      //monitor.{{stakeholderType}}.{{queryname}}
      //parameters: stakeholderId*, type
      if (query["query"]["name"]) {
        name = query["query"]["name"];
        let parameters = (query["query"]["parameters"])?query["query"]["parameters"]:[];
        if(name.split('.')[0] == "rcd" && parameters.length > 0  && stakeholder.type=="ri") {
          //rcd.{{queryname}}
          parameters[0] = ChartHelper.prefix + "index_id" + ChartHelper.suffix;
          indicatorPath.parameters["index_id"] = stakeholder.index_id;
        }else if(name.split('.')[0] == "monitor" && parameters.length == 0 && stakeholder.type=="funder"){
          // old saved queries without params
          //monitor.{{funder_shortName}}.{{type}}.{{queryname}}
          let stakeholderSN = name.split('.')[1];
          query["query"]["name"] = name.split('.' + stakeholderSN + ".")[0] + "." + ChartHelper.prefix + "index_shortName" + ChartHelper.suffix +"." + name.split('.' + stakeholderSN + ".")[1];
          indicatorPath.parameters["index_shortName"] = stakeholder.index_shortName.toLowerCase();
        }else if(name.split('.')[0] == "monitor" && parameters.length > 0 && name.split('.')[1]  == stakeholder.type) {
          // new parameterized queries
          //monitor.{{type}}.{{queryname}}.{{param1 - id }}.{{param2 result-type}}.{{fl0}}
          if( name.split('.').length > 3 && name.split('.')[3] == "id") {
            parameters[0] = ChartHelper.prefix + "index_id" + ChartHelper.suffix;
            indicatorPath.parameters["index_id"] = stakeholder.index_id;
          }else if( name.split('.').length > 3 && name.split('.')[3] == "shortname") {
            parameters[0] = ChartHelper.prefix + "index_shortName" + ChartHelper.suffix;
            indicatorPath.parameters["index_shortName"] = stakeholder.index_shortName.toLowerCase();
          }else if( name.split('.').length > 3 && name.split('.')[3] == "name") {
            parameters[0] = ChartHelper.prefix + "index_name" + ChartHelper.suffix;
            indicatorPath.parameters["index_name"] = stakeholder.index_name;
          }
        }
      }
    }
  }
  private extractDataTitle(obj, indicatorPath: IndicatorPath) {
    let index = 0;
    if(!obj[this.getDescriptionObjectName(obj)] || !obj[this.getDescriptionObjectName(obj)][this.getQueryObjectName(obj)]){
      return;
    }
    for (let query of obj[this.getDescriptionObjectName(obj)][this.getQueryObjectName(obj)]) {
      if (query["name"]) {
        let name = query["name"];
        query["name"] =  ChartHelper.prefix + "data_title_"+index + ChartHelper.suffix;
        indicatorPath.parameters["data_title_"+index] = name;
      }
      index++;
    }
  }
  private extractTitle(obj, indicatorPath: IndicatorPath) {
    let title = "";
    if (obj[this.getDescriptionObjectName(obj)]["title"]) {
      title = obj[this.getDescriptionObjectName(obj)]["title"]["text"];
      obj[this.getDescriptionObjectName(obj)]["title"]["text"] = ChartHelper.prefix + "title" + ChartHelper.suffix;
    }else if (obj[this.getDescriptionObjectName(obj)]["options"] && obj[this.getDescriptionObjectName(obj)]["options"]["title"]) {
      title = obj[this.getDescriptionObjectName(obj)]["options"]["title"];
      obj[this.getDescriptionObjectName(obj)]["options"]["title"] = ChartHelper.prefix + "title" + ChartHelper.suffix;
    }
    indicatorPath.parameters["title"] = title ? title : "";
  }

  private extractSubTitle(obj, indicatorPath: IndicatorPath) {
    let subtitle = "";
    if (obj[this.getDescriptionObjectName(obj)]["subtitle"]) {
      subtitle = obj[this.getDescriptionObjectName(obj)]["subtitle"]["text"];
      obj[this.getDescriptionObjectName(obj)]["subtitle"]["text"] = ChartHelper.prefix + "subtitle" + ChartHelper.suffix;
      indicatorPath.parameters["subtitle"] = subtitle ? subtitle : "";
    }else if (obj[this.getDescriptionObjectName(obj)]["title"] && obj[this.getDescriptionObjectName(obj)]["title"] && obj[this.getDescriptionObjectName(obj)]["title"]["subtext"]) {
      subtitle = obj[this.getDescriptionObjectName(obj)]["title"]["subtext"];
      obj[this.getDescriptionObjectName(obj)]["title"]["subtext"] = ChartHelper.prefix + "subtitle" + ChartHelper.suffix;
      indicatorPath.parameters["subtitle"] = subtitle ? subtitle : "";
    }
  }

  private extractXTitle(obj, indicatorPath: IndicatorPath) {
    let title = "";
    if (obj[this.getDescriptionObjectName(obj)]["xAxis"] && obj[this.getDescriptionObjectName(obj)]["xAxis"]["title"]) {
      title = obj[this.getDescriptionObjectName(obj)]["xAxis"]["title"]["text"];
      obj[this.getDescriptionObjectName(obj)]["xAxis"]["title"]["text"] = ChartHelper.prefix + "xAxisTitle" + ChartHelper.suffix;
    }else if (obj[this.getDescriptionObjectName(obj)]["options"] && obj[this.getDescriptionObjectName(obj)]["options"]["hAxis"] && obj[this.getDescriptionObjectName(obj)]["options"]["hAxis"]["title"]) {
      title = obj[this.getDescriptionObjectName(obj)]["options"]["hAxis"]["title"];
      obj[this.getDescriptionObjectName(obj)]["options"]["hAxis"]["title"] = ChartHelper.prefix + "xAxisTitle" + ChartHelper.suffix;
    }else if (obj[this.getDescriptionObjectName(obj)]["xAxis"] && obj[this.getDescriptionObjectName(obj)]["xAxis"]["name"]) {
      title = obj[this.getDescriptionObjectName(obj)]["xAxis"]["name"];
      obj[this.getDescriptionObjectName(obj)]["xAxis"]["name"] = ChartHelper.prefix + "xAxisTitle" + ChartHelper.suffix;
    }
    indicatorPath.parameters["xAxisTitle"] = title ? title : "";
  }

  private extractYTitle(obj, indicatorPath: IndicatorPath) {
    let title = "";
    if (obj[this.getDescriptionObjectName(obj)]["yAxis"] && obj[this.getDescriptionObjectName(obj)]["yAxis"]["title"] ) {
      title = obj[this.getDescriptionObjectName(obj)]["yAxis"]["title"]["text"];
      obj[this.getDescriptionObjectName(obj)]["yAxis"]["title"]["text"] = ChartHelper.prefix + "yAxisTitle" + ChartHelper.suffix;
    }else if (obj[this.getDescriptionObjectName(obj)]["options"]&& obj[this.getDescriptionObjectName(obj)]["options"]["vAxis"] && obj[this.getDescriptionObjectName(obj)]["options"]["vAxis"]["title"]) {
      title = obj[this.getDescriptionObjectName(obj)]["options"]["vAxis"]["title"];
      obj[this.getDescriptionObjectName(obj)]["options"]["vAxis"]["title"] = ChartHelper.prefix + "yAxisTitle" + ChartHelper.suffix;
    }else if (obj[this.getDescriptionObjectName(obj)]["yAxis"] && obj[this.getDescriptionObjectName(obj)]["yAxis"]["name"]) {
      title = obj[this.getDescriptionObjectName(obj)]["yAxis"]["name"];
      obj[this.getDescriptionObjectName(obj)]["yAxis"]["name"] = ChartHelper.prefix + "xAxisTitle" + ChartHelper.suffix;
    }
    indicatorPath.parameters["yAxisTitle"] = title ? title : "";
  }

  private extractOldToolTitle(obj, indicatorPath: IndicatorPath) {
    let title = "";
    if (obj["title"]) {
      title = obj["title"];
      obj["title"] = ChartHelper.prefix + "title" + ChartHelper.suffix;
      indicatorPath.parameters["title"] = title;

    }
  }

  private extractOldToolXTitle(obj, indicatorPath: IndicatorPath) {
    let title = "";
    if (obj["xaxistitle"]) {
      title = obj["xaxistitle"];
      obj["xaxistitle"] = ChartHelper.prefix + "xAxisTitle" + ChartHelper.suffix;
      indicatorPath.parameters["xAxisTitle"] = title;
    }
  }

  private extractOldToolYTitle(obj, indicatorPath: IndicatorPath) {
    let title = "";
    if (obj["fieldsheaders"]) {
      title = Array.isArray(obj["fieldsheaders"]) ? obj["fieldsheaders"][0] : obj["fieldsheaders"];
      if (Array.isArray(obj["fieldsheaders"])) {
        obj["fieldsheaders"][0] = ChartHelper.prefix + "yAxisTitle" + ChartHelper.suffix;
      } else {
        obj["fieldsheaders"] = ChartHelper.prefix + "yAxisTitle" + ChartHelper.suffix;
      }
      indicatorPath.parameters["yAxisTitle"] = title;
    }
  }

}
